﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.WebLoadTest.Utilities;
using Microsoft.VisualStudio.TestTools.WebTesting;
using System.Collections.Generic;

namespace Benday.WebLogTest.UnitTests
{
    [TestClass]
    public class MathExtractionRuleFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            _ContextValueProvider = null;
        }
        
        private MathExtractionRule _SystemUnderTest;
        public MathExtractionRule SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new MathExtractionRule(
                        ContextValueProvider);
                }

                return _SystemUnderTest;
            }
        }

        private StubContextValueProvider _ContextValueProvider;
        public StubContextValueProvider ContextValueProvider
        {
            get
            {
                if (_ContextValueProvider == null)
                {
                    _ContextValueProvider = new StubContextValueProvider();
                }

                return _ContextValueProvider;
            }
        }

        private void InitializeSystemUnderTest(string value1, 
            string operation, 
            string value2)
        {
            SystemUnderTest.Value1 = value1;
            SystemUnderTest.Value2 = value2;
            SystemUnderTest.Operation = operation;
            SystemUnderTest.ContextParameterName = "output";
        }
                
        [TestMethod]
        public void AddTwoValues()
        {
            string value1 = "1";
            string value2 = "2";
            string expected = "3";

            InitializeSystemUnderTest(value1, "+", value2);
                        
            SystemUnderTest.Extract(null, null);

            AssertResult(expected);
        }

        [TestMethod]
        public void SubtractTwoValues()
        {
            string value1 = "1";
            string value2 = "2";
            string expected = "-1";
                        
            InitializeSystemUnderTest(value1, "-", value2);

            SystemUnderTest.Extract(null, null);

            AssertResult(expected);
        }

        [TestMethod]
        public void MultiplyTwoValues()
        {
            string value1 = "3";
            string value2 = "2";
            string expected = "6";

            InitializeSystemUnderTest(value1, "*", value2);

            SystemUnderTest.Extract(null, null);

            AssertResult(expected);
        }

        [TestMethod]
        public void DivideTwoValues()
        {            
            string value1 = "6";
            string value2 = "3";
            string expected = "2";

            InitializeSystemUnderTest(value1, "/", value2);

            SystemUnderTest.Extract(null, null);

            AssertResult(expected);
        }

        [TestMethod]
        [ExpectedException(typeof(DivideByZeroException))]
        public void DivideTwoValuesThrowsDivideByZeroException()
        {
            string value1 = "6";
            string value2 = "0";

            InitializeSystemUnderTest(value1, "/", value2);

            SystemUnderTest.Extract(null, null);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void WhenJunkValueIsProvidedThrowsAnException()
        {
            string value1 = "asdf";
            string value2 = "0";

            SystemUnderTest.InvalidValuesBecomeZero = false;

            InitializeSystemUnderTest(value1, "+", value2);

            SystemUnderTest.Extract(null, null);
        }

        private void AssertResult(string expected)
        {
            var actual = ContextValueProvider.GetValue("output");

            Assert.IsNotNull(actual, "Result value was null.");

            Assert.AreEqual<string>(expected,
                actual,
                "Result value was wrong.");
        }
    }
}